@@ -9,6 +9,7 @@
 #include "sizeclass.h"
 
 #include <new>
+#include <string.h>
 
 namespace snmalloc
 {
@@ -55,12 +56,71 @@ namespace snmalloc
   // global state of the allocator.  This is currently stored in the memory
   // provider, so we add this in.
   template<class PAL>
-  class MemoryProviderStateMixin : public PAL
+  class MemoryProviderStateMixin : public PalNotificationObject, public PAL
   {
+    /**
+     * Flag to protect the bump allocator
+     **/
     std::atomic_flag lock = ATOMIC_FLAG_INIT;
-    void* bump;
-    size_t remaining;
 
+    /**
+     * Pointer to block being bump allocated
+     **/
+    void* bump = nullptr;
+
+    /**
+     * Space remaining in this block being bump allocated
+     **/
+    size_t remaining = 0;
+
+    /**
+     * Simple flag for checking if another instance of lazy-decommit is
+     * running
+     **/
+    std::atomic_flag lazy_decommit_guard = {};
+
+  public:
+    /**
+     * Stack of large allocations that have been returned for reuse.
+     */
+    ModArray<NUM_LARGE_CLASSES, MPMCStack<Largeslab, RequiresInit>> large_stack;
+
+    /**
+     * Make a new memory provide for this PAL.
+     **/
+    static MemoryProviderStateMixin<PAL>* make() noexcept
+    {
+      // Temporary stack-based storage to start the allocator in.
+      MemoryProviderStateMixin<PAL> local;
+
+      // Allocate permanent storage for the allocator usung temporary allocator
+      MemoryProviderStateMixin<PAL>* allocated =
+        local.alloc_chunk<MemoryProviderStateMixin<PAL>, 1>();
+
+#ifdef GCC_VERSION_EIGHT_PLUS
+#  pragma GCC diagnostic push
+#  pragma GCC diagnostic ignored "-Wclass-memaccess"
+#endif
+      // Put temporary allocator we have used, into the permanent storage.
+      // memcpy is safe as this is entirely single threaded: the move
+      // constructors were removed as unsafe to move std::atomic in a
+      // concurrent setting.
+      memcpy(allocated, &local, sizeof(MemoryProviderStateMixin<PAL>));
+#ifdef GCC_VERSION_EIGHT_PLUS
+#  pragma GCC diagnostic pop
+#endif
+
+      // Register this allocator for low-memory call-backs
+      if constexpr (pal_supports<LowMemoryNotification, PAL>)
+      {
+        allocated->PalNotificationObject::pal_notify = &(allocated->process);
+        PAL::register_for_low_memory_callback(allocated);
+      }
+
+      return allocated;
+    }
+
+  private:
     void new_block()
     {
       // Reserve the smallest large_class which is SUPERSLAB_SIZE
@@ -71,11 +131,6 @@ namespace snmalloc
       remaining = SUPERSLAB_SIZE;
     }
 
-    /**
-     * The last time we saw a low memory notification.
-     */
-    std::atomic<uint64_t> last_low_memory_epoch = 0;
-    std::atomic_flag lazy_decommit_guard;
     SNMALLOC_SLOW_PATH void lazy_decommit()
     {
       // If another thread is try to do lazy decommit, let it continue.  If
@@ -140,12 +195,17 @@ namespace snmalloc
       large_stack[large_class].push(reinterpret_cast<Largeslab*>(p));
     }
 
-  public:
-    /**
-     * Stack of large allocations that have been returned for reuse.
-     */
-    ModArray<NUM_LARGE_CLASSES, MPMCStack<Largeslab, PreZeroed>> large_stack;
+    /***
+     * Method for callback object to perform lazy decommit.
+     **/
+    static void process(PalNotificationObject* p)
+    {
+      // Unsafe downcast here. Don't want vtable and RTTI.
+      auto self = reinterpret_cast<MemoryProviderStateMixin<PAL>*>(p);
+      self->lazy_decommit();
+    }
 
+  public:
     /**
      * Primitive allocator for structure that are required before
      * the allocator can be running.
@@ -197,24 +257,6 @@ namespace snmalloc
       return new (p) T(std::forward<Args...>(args)...);
     }
 
-    /**
-     * Returns the number of low memory notifications that have been received
-     * (over the lifetime of this process).  If the underlying system does not
-     * support low memory notifications, this will return 0.
-     */
-    SNMALLOC_FAST_PATH
-    uint64_t low_memory_epoch()
-    {
-      if constexpr (pal_supports<LowMemoryNotification, PAL>)
-      {
-        return PAL::low_memory_epoch();
-      }
-      else
-      {
-        return 0;
-      }
-    }
-
     template<bool committed>
     void* reserve(size_t large_class) noexcept
     {
@@ -271,8 +313,6 @@ namespace snmalloc
           }
         }
 
-        // printf("Alloc %zx (size = %zx)\n", start, size);
-
         void* result = pointer_cast<void>(start);
         if (committed)
           PAL::template notify_using<NoZero>(result, size);
@@ -280,41 +320,6 @@ namespace snmalloc
         return result;
       }
     }
-
-    SNMALLOC_FAST_PATH void lazy_decommit_if_needed()
-    {
-#ifdef TEST_LAZY_DECOMMIT
-      static_assert(
-        TEST_LAZY_DECOMMIT > 0,
-        "TEST_LAZY_DECOMMIT must be a positive integer value.");
-      static std::atomic<uint64_t> counter;
-      auto c = counter++;
-      if (c % TEST_LAZY_DECOMMIT == 0)
-      {
-        lazy_decommit();
-      }
-#else
-      if constexpr (decommit_strategy == DecommitSuperLazy)
-      {
-        auto new_epoch = low_memory_epoch();
-        auto old_epoch = last_low_memory_epoch.load(std::memory_order_acquire);
-        if (new_epoch > old_epoch)
-        {
-          // Try to update the epoch to the value that we've seen.  If
-          // another thread has seen a newer epoch than us (or done the same
-          // update) let them win.
-          do
-          {
-            if (last_low_memory_epoch.compare_exchange_strong(
-                  old_epoch, new_epoch))
-            {
-              lazy_decommit();
-            }
-          } while (old_epoch < new_epoch);
-        }
-      }
-#endif
-    }
   };
 
   using Stats = AllocStats<NUM_SIZECLASSES, NUM_LARGE_CLASSES>;
@@ -345,7 +350,6 @@ namespace snmalloc
         size = rsize;
 
       void* p = memory_provider.large_stack[large_class].pop();
-      memory_provider.lazy_decommit_if_needed();
 
       if (p == nullptr)
       {
@@ -356,8 +360,7 @@ namespace snmalloc
       {
         stats.superslab_pop();
 
-        // Cross-reference alloc.h's large_dealloc decommitment condition
-        // and lazy_decommit_if_needed.
+        // Cross-reference alloc.h's large_dealloc decommitment condition.
         bool decommitted =
           ((decommit_strategy == DecommitSuperLazy) &&
            (static_cast<Baseslab*>(p)->get_kind() == Decommitted)) ||
@@ -413,7 +416,6 @@ namespace snmalloc
 
       stats.superslab_push();
       memory_provider.large_stack[large_class].push(static_cast<Largeslab*>(p));
-      memory_provider.lazy_decommit_if_needed();
     }
   };
 
@@ -422,5 +424,8 @@ namespace snmalloc
    * The memory provider that will be used if no other provider is explicitly
    * passed as an argument.
    */
-  inline GlobalVirtual default_memory_provider;
+  inline GlobalVirtual& default_memory_provider()
+  {
+    return *(Singleton<GlobalVirtual*, GlobalVirtual::make>::get());
+  }
 } // namespace snmalloc
